package listFW.visitor;
import listFW.*;

/**
 * Inserts an Integer into an ordered host list, assuming the host list contains
 * only Integer objects.  Has no knowledge of how IList is implemented.  Must
 * make use of a list factory (IListFactory) to create IList objects instead of
 * calling the constructors of concrete subclasses directly.
 */
public class InsertInOrderWithFactory implements IListAlgo {

    private IListFactory _listFact;

    public InsertInOrderWithFactory(IListFactory lf) {
        _listFact = lf;
    }

    /**
     * Simply makes a new non-empty list with the given inp parameter as first.
     * @param host an empty IList.
     * @param inp an Integer to be inserted in order into host.
     */
    public Object emptyCase(IMTList host, Object inp) {
        return _listFact.makeNEList(inp, host);
    }

    /**
     * Recur!
     * @param host a non-empty IList.
     * @param inp an Integer to be inserted in order into host.
     */
    public Object nonEmptyCase(INEList host, Object inp) {
        int n = ((Integer)inp).intValue();
        int f = ((Integer)host.getFirst()).intValue();
        return n < f ?
                _listFact.makeNEList(inp, host):
                _listFact.makeNEList(host.getFirst(),
                                    (IList)host.getRest().execute(this, inp));
    }
}